/****************************************************************
* fires:	FInds REpeated Sequences in a (meta)genome. Sequence must
* be in fasta format (can read multiple fasta sequences in the same file)
* Output corresponds to the number of repeatitions for each position
* in the genome. Word size is selectable and it finds both the identical sequence
* and its reverse complementary.
*
* (C) Copyright. All rights reserved. Juan M. Gonzalez. 2007
* Permission to use, reproduce, and distribute this program
* should be granted by the author.
*
* Build: gcc -o fires fires.c
*
* Date: 9/30/2007, 2/9/2008, 2/18/08, 5/24/08    
*
* Author: Juan M. Gonzalez
*
* Address: IRNAS-CSIC,
*          Avda. Reina Mercedes 10
*          41012 - Sevilla, Spain
* e-mail:  jmgrau@irnase.csic.es
*
*****************************************************************/
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>     /* standard ANSI only */
#include <string.h>
#include <malloc.h>

/*---------------------------------------------------*/

#define MAX_LINE 501		/* maximum length of an input line of data */
#define MAX_LINE_WRITE 60	/* line length for writing the output file */
#define MAX_SITE 50		/* maximum legth for a restriction site or word */

/*---------------------------------------------------*/

     /* global declarations affected by the command-line options */
int screen;		/* controls output list to screen */
int copyright;          /* Printing copyright; -c */
int print_all;          /* signals if every word and their number of repeatitions are to be printed */
int mismatch;		/* selects the level of mismatch allowed in each word comparison */

/*---------------------------------------------------*/

main(int argc, char *argv[])
{

        /* variables */
        char *program_name;     /* name of the program collected from
                                   the command line (for errors?) */
        char file_in1[25];      /* name of file with the input sequence */
        char file_out1[25];     /* file name to output all the result */
	char file_out2[25];	/* file name 2 with the most frequent repeatitions */
        FILE *data_file_in1;    /* file to read the sequences to check for repeations */
        FILE *data_file_out1;   /* file to write the results */
	FILE *data_file_out2;	/* file to write the most abundant repeatitions */
	int i; 			/* counter */
        int bl;			/* counter for the number of bases per line */ 
	int bw;			/* counter per bases of word */
	char line1[MAX_LINE];    /* array to read each input line from file_in1 */ 
	char *data_ptr;	        /* character returned by fgets. NULL= end of file */ 
	long base;		/* counts of the number of bases in the sequence */ 
	int selects;		/* selects repeatitions higher than this number */
	long rep;		/* counts for number of repeatitions */
	long rep_rc;		/* counts number of repeatitions and the RC */
	char word[MAX_SITE];	/* fragment to be found as repeated */
	int sequence = 0;	/* number of sequences read from the input file */
	int word_size;		/* length of word */
	char *seq;		/* mantains the full sequence/s (genome) from file 1 */
	int *b_seq;		/* number of bases per sequence if multiple sequences are processed */
	long length, b_length;	/* total sequences length + one ">" to indicate each sequence beginning */
	long previous; 		/* previous base processed for printing */
	int num_seqs;		/* number of sequences in input file */

                /* declaration of functions */
        void usage(char *program_name);
        void search_string(char *seq, long length, char word[], long *rep_ptr, long *rep_rc_ptr, int mismatch);
        void print_header(FILE *data_file_out1_ptr, FILE *data_file_out2_ptr, char file_in1[], int word_size, char *program_name, int print_all);
        void print_cr(void);

/* ---------------------------------------------------------------- */
	/* variables affected by command-line options. Default values */
	screen = 0;	/* Output list of fragments to screen or video;
			   Default is not, 0; screen = 1 output list */
	print_all = 0;	/* Print every word and its number of repeatitions */
	copyright = 1;	/* If equals 0, copyright will not be printed. If equals 1 then copyright information will be printed */
	mismatch = 0;	/* Number of mismatches (number of nucleotides) allowed. Default = 1 */

/* ---------------------------------------------------------------- */

        /* command line arguments stuff */
        program_name = argv[0];         /* for errors and usage function */

        if (argc == 1) {
                (void)usage(program_name);
                exit(8);
        }
        while ((argc>1) && (argv[1][0] == '-')) {

                switch (argv[1][1]) {

                          /*
                          *  -I<file_name> input file name 1 
                          *    argv[1][0] is the dash
                          *    argv[1][1] is the 'I'
                          *    argv[1][2] starts the name
                          */
                        case 'I':
                          (void)strcpy(file_in1, &argv[1][2]);
                          break;

                          /*
                          *  -O<file_name> output 1 file name
                          *    argv[1][0] is the dash
                          *    argv[1][1] is the 'O'
                          *    argv[1][2] starts the name
                          */
                        case 'O':
                          (void)strcpy(file_out1, &argv[1][2]);
			  print_all = 1;
                          break;

                         /*
                          *  -o<file_name> output 2 file name
                          *    argv[1][0] is the dash
                          *    argv[1][1] is the 'o'
                          *    argv[1][2] starts the name
                          */
                        case 'o':
                          (void)strcpy(file_out2, &argv[1][2]);
                          break;

                         /*
                          *  -W<integer> word length
                          *    argv[1][0] is the dash
                          *    argv[1][1] is the 'W'
                          *    argv[1][2] integer
                          */
                        case 'W':
                          word_size = atoi(&argv[1][2]);
                          break;

			 /*
                          *  -S<integer> selects repeatitions higher than integer
                          *    argv[1][0] is the dash
                          *    argv[1][1] is the 'S'
                          *    argv[1][2] integer
			  */
                        case 'S':
			  selects = atoi(&argv[1][2]);
                          break;

                         /*
                          *  -M<integer> selects the level of mismatches to be permitted
                          *    argv[1][0] is the dash
                          *    argv[1][1] is the 'M'
                          *    argv[1][2] integer
                          */
                        case 'M':
                          mismatch = atoi(&argv[1][2]);
                          break;

                          /*
                          *  -c avoid printing the copyright information
                          *    argv[1][0] is the dash
                          *    argv[1][1] is the 'c'
                          *    argv[1][2] no use of any filename or number
                          */
                        case 'c':
                          copyright = 0;
                          break;

			  /*
			  *   -h displays help
			  *     argv[1][0] is the dash
			  *     argv[1][1] is the 'h'
			  *     argv[1][2] not used
			  */
			case 'h':
			  (void)usage(program_name);
			  break;

               /* insert other options for the command line here */
 
                        default:
                          (void)fprintf(stderr, "Bad option %s\n", argv[1]);  
                          (void)usage(program_name);
                }  /* end of switch */

                /* move the argument list up one, move the count down one */
                argc--;
                argv++;
        }       /* end of while */

/* ========================================================================== */

        /* print copyright */
        if (copyright == 1) {
                (void)print_cr();
        }

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

        /* initialize the output and input files */
        data_file_in1 = fopen(file_in1, "r");
        /* check if the file can be open */
        if (file_in1 == NULL) {
                (void)fprintf(stderr, "Sequence file could not be opened.\n \n");
                exit(8);
        }

	/* Pre-read input file 1 to calculate the size of memory to be allocated */
	num_seqs = 0;
	length = 0;
        while (1) {  /* pick words along sequence in file1 */
                /* read one line at a time */
                data_ptr = fgets(line1, sizeof(line1), data_file_in1);
                /* check if we read every line from input 1 */
                if (data_ptr == NULL) {
                        /* end of file. Stop searching */
                        break;
                }
                else if (line1[0] == '>') {
                        /* This is the header line from input fasta file */
                        /* initialize counters for new sequence */
        		num_seqs = num_seqs + 1;
                }
                else if (line1[0] != '>') {
                        /* Process current line of sequence */
                        bl = 0;
                        /* process bases in current line */
                        while ((line1[bl] != '\0') && (line1[bl] != '\n')) {
				length = length + 1;
				bl = bl + 1;
                        }
                }
        }
	length = length + num_seqs - 1;  /* a ">" will be added before every new sequence */
        /* close files, we are done with them */
        (void)fclose(data_file_in1);

	/* allocate memory for holding sequence information */
        seq = (char *)malloc(length * sizeof(char));
        assert(seq != NULL);
        b_seq = (int *)malloc(num_seqs * sizeof(int));
        assert(b_seq != NULL);
	
	/* open input file 1 to read sequence */
        data_file_in1 = fopen(file_in1, "r");

	length = 0;		     /* total number of bases + ">" to indicate the beginning of each sequence */
	num_seqs = 0;		     /* number of sequences */
        base = 0;                    /* initialize to 0 the number of bases */
        bl = 0;                      /* number of bases processed in current line */
        /* Read file1 */
        while (1) {  /* pick words along sequence in file1 */

                /* read one line at a time */
                data_ptr = fgets(line1, sizeof(line1), data_file_in1);
                /* check if we read every line from input 1 */
                if (data_ptr == NULL) {
                        /* end of file. Stop searching */
			b_seq[num_seqs - 1] = base;
                        break;
                }
                else if (line1[0] == '>') {
                        /* This is the header line from input fasta file */
			/* finish with previous sequence */
			if (num_seqs > 0) b_seq[num_seqs - 1] = base;
                        /* initialize counters for new sequence */
                        bl = 0;
                        base = 0;
			seq[length] = '>';   /* start sequence with ">" */
			num_seqs = num_seqs + 1;
			length = length + 1;
                }
                else if (line1[0] != '>') {
                        /* Process current line of sequence */
                        bl = 0;
                        /* process bases in current line */
                        while ((line1[bl] != '\0') && (line1[bl] != '\n')) {
				seq[length] = line1[bl];
				length = length + 1;
                                bl = bl + 1;
                                base = base + 1;
                        }
                }
        }
        /* close input file */
        (void)fclose(data_file_in1);

	/* Now, "seq" holds the whole sequence including ">" to signal the beginning of each sequence
	   and "b_seq" mantains the length of each sequence */ 

        /* open input file 1 to read words in sequence */
        data_file_in1 = fopen(file_in1, "r");
	/* open files for output */
        if (print_all > 0) data_file_out1 = fopen(file_out1, "w");
        data_file_out2 = fopen(file_out2, "w");
	/* print header for output files */
        (void)print_header(data_file_out1, data_file_out2, file_in1, word_size, program_name, print_all);

        b_length = 0;                /* total number of bases + ">" to indicate the beginning of each sequence */
        num_seqs = 0;                /* number of sequences */
        base = 0;                    /* initialize to 0 the number of bases */
        bl = 0;                      /* number of bases processed in current line */
	bw = 0;			     /* counting bases in word */
	/* read words base by base */
        while (1) {  /* pick words along sequence in file1 */

		/* Inform on current approximated location */
		if (base > 0)
                (void)fprintf(stdout, "Working around base %ld (Seq. %d).\n", base+1, num_seqs);

                /* read one line at a time */
                data_ptr = fgets(line1, sizeof(line1), data_file_in1);
                /* check if we read every line from input 1 */
                if (data_ptr == NULL) {
                        /* end of file. Stop searching */
                        break;
                }
                else if (line1[0] == '>') {
                        /* This is the header line from input fasta file */
                        /* initialize counters for new sequence */
                        bw = 0;
                        bl = 0;
                        base = 0;
			previous = 0;
                        if (print_all > 0) (void)fprintf(data_file_out1, ">Seq No. %d. Length %d\n", num_seqs + 1, b_seq[num_seqs]);
                        (void)fprintf(data_file_out2, ">Seq No. %d. Length %d\n", num_seqs + 1, b_seq[num_seqs]);
			num_seqs = num_seqs + 1;
                }
                else if (line1[0] != '>') {
                        /* Process current line of sequence */
                        bl = 0;
                        /* process bases in current line */
                        while ((line1[bl] != '\0') && (line1[bl] != '\n')) {
                                word[bw] = line1[bl];
                                if (bw >= (word_size - 1)) {
                                        /* search word */
                                        search_string(seq, length, word, &rep, &rep_rc, mismatch);
                                        /* print results */
                                        if (print_all > 0) {
                                                (void)fprintf(data_file_out1, "%ld %s %d %d %d \n", base+1, word, rep, rep_rc, (rep_rc+rep));
                                        }
                                        if ((rep_rc + rep) >= selects) {
						if ((previous + 1) < base) {
							(void)fprintf(data_file_out2, "%ld NONE %d %d %d \n", base, 1, 0, 1);	
						}
                                                (void)fprintf(data_file_out2, "%ld %s %d %d %d \n", base+1, word, rep, rep_rc, (rep_rc+rep));
						previous = base;
                                        }
					else {
						if ((previous + 1) == base) {
                                                (void)fprintf(data_file_out2, "%ld NONE %d %d %d \n", base+1, 1, 0, 1);
						}	
					}
                                        for (i = 1; i <= (word_size - 1); i++) {
                                                word[i - 1] = word[i];
                                        }
                                        bw = bw -1;
                                }
                                bw = bw + 1;
                                bl = bl + 1;
                                base = base + 1;
				b_length = b_length + 1;
                        }
                }
        }

        /* close files, we are done with them */
        (void)fclose(data_file_in1);
	if (print_all > 0) (void)fclose(data_file_out1);
        (void)fclose(data_file_out2);

	/* liberate allocated memory */
        free(seq);
	free(b_seq);

        return;

}

/****************************************************************
Searches for the Word in the sequence of filein2 and reports
the number of repeatitions and the total counting lso its RC
*****************************************************************/
void search_string(char *seq,
		   long length,
		   char word[MAX_SITE],
		   long *rep_ptr,
		   long *rep_rc_ptr,
		   int  mismatch)
{
	int found_site;		/* signals if a site has been found */
	char re_site[MAX_SITE]; /* Word */
	char line2[MAX_LINE];	/* line from input file */
	int num_seqs2 = 0;	/* number of sequences */
	long s;			/* counts the number of bases per sequence */
	int i;			/* counter */
	long j;			/* sequences in "seq" */
	int i_res;		/* length of word */
	int b_rf;		/* counter bases per word */
	char rf[MAX_SITE];	/* fragment to be compared to word */

        void find_site(int *found_site_ptr, char re_site[], int i_res, char tmp_rf[], int mismatch);

	i_res = strlen(word); /* equals to word_size */
	for (i = 0; i < i_res; i++) {
		re_site[i] = word[i];	
	}

	s = 0;		/* bases in each sequence */
	b_rf = 0;	/* bases in word or restriction fragment */
	*rep_ptr = 0;	/* number of repeatitions of word */
	num_seqs2 = 0;  /* number of sequences in global sequence file */
        /* read sequence from memory to screen for each word */
        for (j = 0; j <= length; j++) {  /* reads sequence from memory */
                if (seq[j] == '>') {
                        /* A new sequence starts */
                        /* initialize counters for new sequence */
                        b_rf = 0;
                        s = 0;
                        found_site = 0;
                        num_seqs2 = num_seqs2 + 1;
                }
                else if (seq[j] != '>') {
                        /* Process current sequence */
                        rf[b_rf] = seq[j];
                        if (b_rf >= (i_res - 1)) {
                            /* find word */
                                find_site(&found_site, re_site, i_res, rf, mismatch);
                                if (found_site == 1) {
                                        *rep_ptr = *rep_ptr + 1;
                                }
                                found_site = 0;  /* initialize for next base */
                                for (i = 1; i <= (i_res - 1); i++) {
                                        rf[i-1] = rf[i];
                                }
                                b_rf = b_rf -1;
                        }
                        b_rf = b_rf + 1;
                        s = s + 1;
                }
	}

	/* Now, start analyzing the reverse complementary */
	/* get the reverse complementary of word */
	for (i = 0; i < i_res; i++) {
		re_site[i] = word[i_res - i - 1];
		if (re_site[i] == 'A') {
			re_site[i] = 'T';
		}
		else if (re_site[i] == 'T') {
			re_site[i] = 'A';
		}
		else if (re_site[i] == 'C') {
			re_site[i] = 'G';
		}
		else if (re_site[i] == 'G') {
			re_site[i] = 'C';
		}
		/* this can be improved for ambiguities */
		else {
			re_site[i] = 'N';
		}
	}

        s = 0;          /* bases in each sequence */
        b_rf = 0;       /* bases in word or restriction fragment */
        *rep_rc_ptr = 0;   /* number of repeatitions of word */
        num_seqs2 = 0;  /* number of sequences in global sequence file */
        /* read sequence from memory to screen for each word */
        for (j = 0; j <= length; j++) {  /* reads sequence from memory */
                if (seq[j] == '>') {
                        /* A new sequence starts */
                        /* initialize counters for new sequence */
                        b_rf = 0;
                        s = 0;
                        found_site = 0;
                        num_seqs2 = num_seqs2 + 1;
                }
                else if (seq[j] != '>') {
                        /* Process current sequence */
                        rf[b_rf] = seq[j];
                        if (b_rf >= (i_res - 1)) {
                               /* find word */
                               find_site(&found_site, re_site, i_res, rf, mismatch);
                               if (found_site == 1) {
                                        *rep_rc_ptr = *rep_rc_ptr + 1;
                               }
                               found_site = 0;  /* initialize for next base */
                               for (i = 1; i <= (i_res - 1); i++) {
                                        rf[i-1] = rf[i];
                               }
                               b_rf = b_rf -1;
                        }
                        b_rf = b_rf + 1;
                        s = s + 1;
                }
        }
/*	*rep_rc_ptr = *rep_rc_ptr + *rep_ptr;
*/

        return;

}

/****************************************************************
Searches for the RE recognition site in the current sequence fragment
It will also process degenerate nucleotides.
*****************************************************************/
void find_site(	int	*found_site_ptr,
		char	re_site[MAX_SITE],
		int	i_res,
		char	tmp_rf[MAX_SITE],
		int     mismatch)
{  
	int i;	/* counter */
	int mm; /* counts mismatches */

/* this code does not check for ambiguous nucleotides */
/*
	*found_site_ptr = 1;
	i = 0;
	while (i < i_res) {
		if (tmp_rf[i] != re_site[i]) {
			*found_site_ptr = 0;
			break;
		}
		i = i + 1;
	}

	return;
}
*/

/* CODE ACCEPTING MISMATCHES. This does not check ambiguous nucleotides */

        *found_site_ptr = 1;
	mm = 0;
        i = 0;
        while (i < i_res) {
                if (tmp_rf[i] != re_site[i]) {
			mm = mm + 1;
			if (mm > mismatch) {
				*found_site_ptr = 0;
				break;
			}
		}
		i = i + 1;
	}

        return;
}



/* CODE FOR AMBIGUOUS NUCLEOTIDEST */
/*
	int ok;

	ok = 0;
        *found_site_ptr = 1;
        i = 0;
        while (i < i_res) {
		if (tmp_rf[i] >>>NOT IN A,T,C,G<<<) {
		    if (tmp_rf[i] != re_site[i]) {
                        if ((re_site[i] == 'Y') && ((tmp_rf[i] == 'C') || (tmp_rf[i] == 'T'))) {
                                ok = 1;
                        }
                        else if ((re_site[i] == 'R') && ((tmp_rf[i] == 'G') || (tmp_rf[i] == 'A'))) {
                                ok = 1;
                        }
                        else if ((re_site[i] == 'S') && ((tmp_rf[i] == 'G') || (tmp_rf[i] == 'C'))) {
                                ok = 1;
                        }
                        else if ((re_site[i] == 'W') && ((tmp_rf[i] == 'T') || (tmp_rf[i] == 'A'))) {
                                ok = 1;
                        }
                        else if ((re_site[i] == 'M') && ((tmp_rf[i] == 'A') || (tmp_rf[i] == 'C'))) {
                                ok = 1;
                        }
                        else if ((re_site[i] == 'K') && ((tmp_rf[i] == 'T') || (tmp_rf[i] == 'G'))) {
                                ok = 1;
                        }
                        else if ((re_site[i] == 'V') && ((tmp_rf[i] == 'A') || (tmp_rf[i] == 'C') || (tmp_rf[i] == 'G'))) {
                                ok = 1;
                        }
                        else if ((re_site[i] == 'H') && ((tmp_rf[i] == 'T') || (tmp_rf[i] == 'A') || (tmp_rf[i] == 'C'))) {
                                ok = 1;
                        }
                        else if ((re_site[i] == 'D') && ((tmp_rf[i] == 'T') || (tmp_rf[i] == 'A') || (tmp_rf[i] == 'G'))) {
                                ok = 1;
                        }
                        else if ((re_site[i] == 'B') && ((tmp_rf[i] == 'T') || (tmp_rf[i] == 'C') || (tmp_rf[i] == 'G'))) {
                                ok = 1;
                        }
                        else if (re_site[i] == 'N') {
                                ok = 1;
                        }
                        else if ((tmp_rf[i] == 'Y') && ((re_site[i] == 'C') || (re_site[i] == 'T'))) {
                                ok = 1;
                        }
                        else if ((tmp_rf[i] == 'R') && ((re_site[i] == 'G') || (re_site[i] == 'A'))) {
                                ok = 1;
                        }
                        else if ((tmp_rf[i] == 'S') && ((re_site[i] == 'G') || (re_site[i] == 'C'))) {
                                ok = 1;
                        }
                        else if ((tmp_rf[i] == 'W') && ((re_site[i] == 'T') || (re_site[i] == 'A'))) {
                                ok = 1;
                        }
                        else if ((tmp_rf[i] == 'M') && ((re_site[i] == 'A') || (re_site[i] == 'C'))) {
                                ok = 1;
                        }
                        else if ((tmp_rf[i] == 'K') && ((re_site[i] == 'T') || (re_site[i] == 'G'))) {
                                ok = 1;
                        }
                        else if ((tmp_rf[i] == 'V') && ((re_site[i] == 'A') || (re_site[i] == 'C') || (re_site[i] == 'G'))) {
                                ok = 1;
                        }
                        else if ((tmp_rf[i] == 'H') && ((re_site[i] == 'T') || (re_site[i] == 'A') || (re_site[i] == 'C'))) {
                                ok = 1;
                        }
                        else if ((tmp_rf[i] == 'D') && ((re_site[i] == 'T') || (re_site[i] == 'A') || (re_site[i] == 'G'))) {
                                ok = 1;
                        }
                        else if ((tmp_rf[i] == 'B') && ((re_site[i] == 'T') || (re_site[i] == 'C') || (re_site[i] == 'G'))) {
                                ok = 1;
                        }
                        else if (tmp_rf[i] == 'N') {
                                ok = 1;
                        }

                   	if (ok == 0) {
                        	*found_site_ptr = 0;
                        	break;
                   	}
		   }
		}
                else if (tmp_rf[i] != re_site[i]) {
                        *found_site_ptr = 0;
                        break;
                }
                i = i + 1;
        }

        return;
}
*/

/****************************************************************
Prints header for output files
*****************************************************************/
void print_header(FILE *data_file_out1_ptr,
		  FILE *data_file_out2_ptr,
                  char file_in1[25],
		  int word_size,
		  char *program_name,
		  int print_all)
{

	int j;	/* counter */

  if (print_all > 0)
  (void)fprintf(data_file_out1_ptr, "#Repeatitions resulting from %s \n", program_name);

  (void)fprintf(data_file_out2_ptr, "#Repeatitions for selected words resulting from %s \n", program_name);

  if (print_all > 0) 
  (void)fprintf(data_file_out1_ptr, "#Input file: %s \n", file_in1);

  (void)fprintf(data_file_out2_ptr, "#Input file: %s \n", file_in1);

  if (print_all > 0)
  (void)fprintf(data_file_out1_ptr, "#Word size: %d nucleotides. \n", word_size);

  (void)fprintf(data_file_out2_ptr, "#Word size: %d nucleotides. \n", word_size);

  if (print_all > 0)
  (void)fprintf(data_file_out1_ptr, "#*********************************************** \n");

  (void)fprintf(data_file_out2_ptr, "#*********************************************** \n");

  if (print_all > 0) {
   (void)fprintf(data_file_out1_ptr, "#Position Word ");
   for (j = 0; j < word_size; j++) {
  	(void)fprintf(data_file_out1_ptr, " ");
   }
   (void)fprintf(data_file_out1_ptr, "Repeatitions RC_repeated Rep.+RCs \n");
  }

  (void)fprintf(data_file_out2_ptr, "#Position Word ");
  for (j = 0; j < word_size; j++) {
        (void)fprintf(data_file_out2_ptr, " ");
  }
  (void)fprintf(data_file_out2_ptr, "Repeatitions RC_repeated Rep.+RCs \n");

  return;

}

/****************************************************************
Print the copyright information.
*****************************************************************/
void print_cr(void)
{

        (void)fprintf(stdout, "\n \n \n *****************************************************************\n \n");
        (void)fprintf(stdout, "    (C) Copyright. All rights reserved. Juan M. Gonzalez. 2007.\n");
        (void)fprintf(stdout, "    Permission to use, reproduce, and distribute this program\n");
        (void)fprintf(stdout, "    or any of its parts should be granted by the author. \n \n");
        (void)fprintf(stdout, "    Juan M. Gonzalez, \n");
        (void)fprintf(stdout, "    IRNASE, P.O.Box 1052, 41080 Sevilla, Spain.\n");
        (void)fprintf(stdout, "    jmgrau@irnase.csic.es \n \n");
        (void)fprintf(stdout, " *****************************************************************\n \n \n");

        return;

}

/****************************************************************
Instructions for command-line arguments and program usage.
Prints the information and exits the program.
*****************************************************************/
void usage(char *program_name)
{
  (void)fprintf(stdout, "\n%s FInds REpeated Sequences in a (meta)genome.\n", program_name);
  (void)fprintf(stdout, "\nUsage is %s -[options]<string or number>\n", program_name); 
  (void)fprintf(stdout, "Sequence must be in fasta format as a file with a single sequence or multiple sequences.\n");
  (void)fprintf(stdout, "Options are:\n");
  (void)fprintf(stdout, "  -I<input file>   Name of input file.\n");
  (void)fprintf(stdout, "  -o<output file>  Name for output file A. Prints only those fragments repeated > S<integer>. Required.\n");
  (void)fprintf(stdout, "  -O<output file>  Name for output file B. Optional. Prints everything, repeated and non-repeated words\n");
  (void)fprintf(stdout, "  -S<integer>      Number of repeatitions. Fragments repeated more than this number will be printed in output file 2.\n");
  (void)fprintf(stdout, "  -W<integer>      Word size to search as number of bases.\n");
  (void)fprintf(stdout, "  -M<integer>      Number of mismatches. Optional. Default 0.\n");
  (void)fprintf(stdout, "  -c               Avoids printing copyright information.\n");
  (void)fprintf(stdout, "Example::\n");
  (void)fprintf(stdout, "	%s -Iinput -ooutput -W25 -S25 -M1 \n \n", program_name);
  exit(8);
}

